@@ -66,7 +66,6 @@ class Server(Base):
     smotd = Column(String(255))
     voice_category_prefix = Column(String(40))
     games = relationship("Game", back_populates="server", uselist=True)
-    channels = relationship("Channel", back_populates="server", uselist=True)
     auto_verify_channels = relationship(
         "AutoVerifyChannel", back_populates="server", uselist=True
     )
@@ -78,11 +77,6 @@ class Server(Base):
     )
     teams = relationship("Team", back_populates="server", uselist=True)
 
-    def bot_allowed_in(self, channel_xid: int) -> bool:
-        return not self.channels or any(
-            channel.channel_xid == channel_xid for channel in self.channels
-        )
-
     def channel_settings_for(self, channel_xid: int) -> Optional[ChannelSettings]:
         filters = {"channel_xid": channel_xid}
         return self.channel_settings.filter_by(**filters).one_or_none()  # type: ignore
@@ -146,7 +140,6 @@ class Server(Base):
                 "prefix": self.prefix,
                 "expire": self.expire,
                 "show_spectate_link": self.show_spectate_link,
-                "channels": [channel.channel_xid for channel in self.channels],
                 "teams": [team.name for team in self.teams],
             }
         )
@@ -184,18 +177,6 @@ class Team(Base):
         return results
 
 
-class Channel(Base):
-    __tablename__ = "channels"
-    channel_xid = Column(BigInteger, primary_key=True, nullable=False)
-    guild_xid = Column(
-        BigInteger,
-        ForeignKey("servers.guild_xid", ondelete="CASCADE"),
-        nullable=False,
-        index=True,
-    )
-    server = relationship("Server", back_populates="channels")
-
-
 class AutoVerifyChannel(Base):
     __tablename__ = "auto_verify_channels"
     channel_xid = Column(BigInteger, primary_key=True, nullable=False)
@@ -233,7 +214,9 @@ class ChannelSettings(Base):
     created_at = Column(DateTime, nullable=False, default=datetime.utcnow)
     updated_at = Column(DateTime, nullable=False, default=datetime.utcnow)
     default_size = Column(Integer, nullable=True)
-    require_verification = Column(Boolean, nullable=False, server_default=false())
+    require_verification = Column(
+        Boolean, nullable=False, server_default=false(), default=False
+    )
     cmotd = Column(String(255))
     verify_message = Column(String(255))
     tags_enabled = Column(Boolean, nullable=True)  # overrides server setting
@@ -466,10 +449,41 @@ class Play(Base):
     )
 
 
+class Award(Base):
+    __tablename__ = "awards"
+    id = Column(Integer, primary_key=True, nullable=False, autoincrement=True)
+    guild_xid = Column(BigInteger, nullable=False)
+    count = Column(Integer, nullable=False)
+    repeating = Column(Boolean, nullable=False, default=False, server_default=false())
+    role = Column(String(60), nullable=False)
+    message = Column(String(255), nullable=False)
+
+
+class UserAward(Base):
+    __tablename__ = "user_awards"
+    user_xid = Column(
+        "user_xid",
+        BigInteger,
+        ForeignKey("users.xid", ondelete="CASCADE"),
+        primary_key=True,
+        nullable=False,
+        index=True,
+    )
+    guild_xid = Column(
+        BigInteger,
+        ForeignKey("servers.guild_xid", ondelete="CASCADE"),
+        primary_key=True,
+        nullable=False,
+        index=True,
+    )
+    current_award_id = Column(Integer, nullable=True)
+    plays = Column(Integer, nullable=False, default=0, server_default=text("0"))
+
+
 class Game(Base):
     __tablename__ = "games"
     id = Column(Integer, primary_key=True, nullable=False, autoincrement=True)
-    created_at = Column(DateTime, nullable=False, default=datetime.utcnow)
+    created_at = Column(DateTime, nullable=False, default=datetime.utcnow, index=True)
     updated_at = Column(DateTime, default=datetime.utcnow, index=True)
     expires_at = Column(DateTime)
     size = Column(Integer, nullable=False, index=True)
@@ -489,7 +503,7 @@ class Game(Base):
     event_id = Column(
         Integer, ForeignKey("events.id", ondelete="SET NULL"), nullable=True, index=True
     )
-    message_xid = Column(BigInteger)
+    message_xid = Column(BigInteger, index=True)
     system = Column(
         String(30), nullable=False, server_default=text("'spelltable'"), index=True
     )
@@ -783,7 +797,7 @@ class Game(Base):
             description += f"\n{self.server.smotd}"
         embed.description = description
 
-        if self.voice_channel_xid:
+        if self.voice_channel_xid and show_link:
             embed.add_field(name="Voice Channel", value=f"<#{self.voice_channel_xid}>")
         tag_names = None
         if self.tags and len(cast(List[Tag], self.tags)) >= 1:
@@ -866,5 +880,5 @@ class Data:
         self.engine = create_engine(db_url, echo=False)
         self.conn = self.engine.connect()
         create_all(self.conn, db_url)
-        self.Session = sessionmaker(bind=self.engine)
+        self.Session = sessionmaker(bind=self.engine, expire_on_commit=True)
         self.metadata = Base.metadata
